/****************************************************************************
** $Id: quickcompiler.cpp  1.1.1   edited 2003-06-20T21:19:30$
**
** Copyright (C) 2001-2003 Trolltech AS.  All rights reserved.
**
** This file is part of the Qt Script for Applications framework (QSA).
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** Licensees holding a valid Qt Script for Applications license may use
** this file in accordance with the Qt Script for Applications License
** Agreement provided with the Software.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
**   information about QSA Commercial License Agreements.
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
*****************************************************************************/

#include "quickcompiler.h"
#include <qfile.h>
#include <qtextstream.h>
#include <qurl.h>
#include <qdir.h>
#include <qimage.h>
//#include <stdio.h>
//#include <stdlib.h>

#if defined(Q_OS_UNIX)
#include <sys/types.h>
#include <sys/stat.h>
#endif

static QStringList getFiles( const QString &proContent, const QString key, const QString & /*extension*/ )
{
    int i = proContent.find( key );
    if ( i != -1 ) {
	QString part = proContent.mid( i + key.length() );
	QStringList lst;
	bool inName = FALSE;
	QString currName;
	for ( i = 0; i < (int)part.length(); ++i ) {
	    QChar c = part[ i ];
	    if ( ( c.isLetter() || c.isDigit() || c == '.' || c == '/' || c == '_' ) &&
		 c != ' ' && c != '\t' && c != '\n' && c != '=' && c != '\\' ) {
		if ( !inName )
		    currName = QString::null;
		currName += c;
		inName = TRUE;
	    } else {
		if ( inName ) {
		    inName = FALSE;
		    if ( currName.simplifyWhiteSpace() != "\\" )
			lst.append( currName );
		}
		if ( c == '\n' && i > 0 && part[ (int)i - 1 ] != '\\' )
		    break;
	    }
	}
	return lst;
    }
    return QStringList();
}

static QStringList uiFiles( const QString &proContent )
{
    QStringList l = getFiles( proContent, "INTERFACES", ".ui" );
    if ( l.isEmpty() )
	l = getFiles( proContent, "FORMS", ".ui" );
    return l;
}


static QStringList sourceFiles( const QString &proContent )
{
    return getFiles( proContent, "DESIGNER_SOURCES", ".qs" );
}

static QString parse_part( const QString &part )
{
    QString res;
    bool inName = FALSE;
    QString currName;
    for ( int i = 0; i < (int)part.length(); ++i ) {
	QChar c = part[ i ];
	if ( !inName ) {
	    if ( c != ' ' && c != '\t' && c != '\n' && c != '=' && c != '\\' )
		inName = TRUE;
	    else
		continue;
	}
	if ( inName ) {
	    if ( c == '\n' )
		break;
	    res += c;
	}
    }
    return res;
}

static QString mainForm( const QString &proContent )
{
    int i = proContent.find( "MAINFORM" );
    if ( i != -1 ) {
	QString mainform = "";
	QString part = proContent.mid( i + QString( "MAINFORM" ).length() );
	mainform = parse_part( part );
	return mainform;
    }
    return "";
}

static QString target( const QString &proContent )
{
    int i = proContent.find( "TARGET" );
    if ( i != -1 ) {
	QString target = "";
	QString part = proContent.mid( i + QString( "TARGET" ).length() );
	target = parse_part( part );
	return target;
    }
    return "";
}

static QString destDir( const QString &proContent )
{
    int i = proContent.find( "DESTDIR" );
    if ( i != -1 ) {
	QString destdir = "";
	QString part = proContent.mid( i + QString( "DESTDIR" ).length() );
	destdir = parse_part( part );
	return destdir;
    }
    return "";
}

static QString appType( const QString &proContent )
{
    int i = proContent.find( "QUICK_APPTYPE" );
    if ( i != -1 ) {
	QString apptype = "GUI";
	QString part = proContent.mid( i + QString( "QUICK_APPTYPE" ).length() );
	apptype = parse_part( part );
	return apptype;
    }
    return "GUI";
}


QuickCompiler::QuickCompiler()
{
}

int QuickCompiler::compile( const QString &i, const QString &dest )
{
    in = i;

    QUrl url( QDir( in ).absPath() );
    QString proContents;
    QFile f( in );
    if ( !f.open( IO_ReadOnly ) ) {
	error( "Couldn't open " + in );
	return -1;
    }

    QTextStream ts2( &f );
    proContents = ts2.read();
    f.close();

    QString out = dest;
    if ( dest.isEmpty() ) {
	out = target( proContents );
	if ( out.isEmpty() ) {
	    error( "No target specified in " + in + " This is required" );
	    return -1;
	}
	if ( out.right( 5 ) != "quick" )
	    out += ".quick";
	QString destdir = destDir( proContents );
	if ( !destdir.isEmpty() ) {
	    out.prepend( destdir + "/" );
	} else {
	    out = QUrl( url.path(), out, TRUE ).path();
	}
    }

    QFile f2( out );
    if ( !f2.open( IO_WriteOnly ) ) {
	error( "Couldn't write " + out );
	return -1;
    }

    QTextStream ts( &f2 );
    //ts << "#!/usr/bin/env quick" << endl;
    ts << "<!DOCTYPE QS><QS version=\"1.0\">" << endl;
    ts << "<mainform>" << mainForm( proContents ) << "</mainform>" << endl;
    ts << "<apptype>" << appType( proContents ) << "</apptype>" << endl;

    int len;
    QString c = compressedContents( in, len );
    ts << "<projectfile file=\"" << QFileInfo( in ).fileName() <<"\" size=\"" << len << "\">" << c << "</projectfile>" << endl;

    QStringList::Iterator it;

    QDir d( QUrl( url.path(), "images", TRUE ).path() );
    if ( !d.exists() || !d.isReadable() ) {
	warning( "Couldn't read " + d.absPath() );
    } else {
	QStringList images = d.entryList( QDir::Files );
	for ( it = images.begin(); it != images.end(); ++it ) {
	    QString fn = d.path() + "/" + *it;
	    int len;
	    QString c = compressedContents( fn, len, TRUE );
	    ts << "<image file=\"" << *it <<"\" size=\"" << len << "\">" << c << "</image>" << endl;
	}
    }

    QStringList sources = sourceFiles( proContents );
    for ( it = sources.begin(); it != sources.end(); ++it ) {
	QString fn = QUrl( url.path(), *it, TRUE ).path();
	int len;
	QString c = compressedContents( fn, len );
	ts << "<sourcefile file=\"" << QFileInfo( *it ).fileName() <<"\" size=\"" << len << "\">" << c << "</sourcefile>" << endl;
    }

    QStringList uifiles = uiFiles( proContents );
    for ( it = uifiles.begin(); it != uifiles.end(); ++it ) {
	QString fn = QUrl( url.path(), *it, TRUE ).path();
	int len;
	QString c = compressedContents( fn + ".qs", len );
	if ( c.isNull() )
	    continue;
	ts << "<formcode file=\"" << QFileInfo( *it ).fileName() << ".qs" << "\" size=\"" << len << "\">" << c << "</formcode>" << endl;
	c = compressedContents( fn, len );
	if ( c.isNull() )
	    continue;
	ts << "<uifile file=\"" << QFileInfo( *it ).fileName() << "\" size=\"" << len << "\">" << c << "</uifile>" << endl;
    }

    ts << "</QS>" << endl;

#if defined(Q_OS_UNIX)
    //chmod( out.latin1(), S_IRUSR | S_IWUSR | S_IXUSR | S_IRGRP | S_IWGRP | S_IXGRP | S_IROTH | S_IXOTH );
#endif

    return 0;
}

QString QuickCompiler::compressedContents( const QString &filename, int &len, bool pixmap )
{
    QString fn( filename );
    QFile f( fn );
    if ( !f.open( IO_ReadOnly ) ) {
	error( "Couldn't read " + fn );
	len = 0;
	return QString::null;
    }

    QByteArray ba;
    if ( !pixmap ) {
	ba = f.readAll();
    } else {
	f.close();
	QImage img( fn );
	fn = QDir::homeDirPath() + "/.tmpquick.xpm";
	img.save( fn, "XPM" );
	f.setName( fn );
	f.open( IO_ReadOnly );
	QTextStream ts( &f );
	ba = ts.read().local8Bit();
    }
    QByteArray bazip = qCompress( ba );
    len = bazip.size();
    QString res;
    static const char hexchars[] = "0123456789abcdef";
    // The first 4 bytes in qCompress() are the length of the unzipped
    // format. The XPM.GZ format does not use these.
    for ( int i = 4; i < (int)len; ++i ) {
	uchar s = (uchar) bazip[i];
	res += hexchars[s >> 4];
	res += hexchars[s & 0x0f];
    }

    return res;
}

void QuickCompiler::error( const QString & /*message*/ )
{
    //fprintf( stdout, "Error: %s\n", message.latin1() );
    //exit( -1 );
}

void QuickCompiler::warning( const QString & /*message*/ )
{
    //fprintf( stdout, "Warning: %s\n", message.latin1() );
}
